@@ -313,6 +313,10 @@ class DifferentialEvolutionProposal(BaseProposal):
 class UniformProposal(BaseProposal):
     """A proposal using uniform draws from the prior support
 
+    Note: for priors with infinite support, this proposal will not propose a
+    point, leading to inefficient sampling. You may wish to omit this proposal
+    if you have priors with infinite support.
+
     Parameters
     ----------
     priors: bilby.core.prior.PriorDict
@@ -330,9 +334,14 @@ class UniformProposal(BaseProposal):
     def propose(self, chain):
         sample = chain.current_sample
         for key in self.parameters:
-            sample[key] = np.random.uniform(
-                self.prior_minimum_dict[key], self.prior_maximum_dict[key]
-            )
+            width = self.prior_width_dict[key]
+            if np.isinf(width) is False:
+                sample[key] = np.random.uniform(
+                    self.prior_minimum_dict[key], self.prior_maximum_dict[key]
+                )
+            else:
+                # Unable to generate a uniform sample on infinite support
+                pass
         log_factor = 0
         return sample, log_factor
 
@@ -921,7 +930,7 @@ def _stretch_move(sample, complement, scale, ndim, parameters):
 
 
 class EnsembleProposal(BaseProposal):
-    """ Base EnsembleProposal class for ensemble-based swap proposals """
+    """Base EnsembleProposal class for ensemble-based swap proposals"""
 
     def __init__(self, priors, weight=1):
         super(EnsembleProposal, self).__init__(priors, weight)
@@ -974,9 +983,12 @@ def get_proposal_cycle(string, priors, L1steps=1, warn=True):
             first_fit=1000, nsamples_for_density=10000, fit_multiplier=2
         )
 
+        all_but_cal = [key for key in priors if "recalib" not in key]
         plist = [
-            AdaptiveGaussianProposal(priors, weight=small_weight),
-            DifferentialEvolutionProposal(priors, weight=small_weight),
+            AdaptiveGaussianProposal(priors, weight=small_weight, subset=all_but_cal),
+            DifferentialEvolutionProposal(
+                priors, weight=small_weight, subset=all_but_cal
+            ),
         ]
 
         if GMMProposal.check_dependencies(warn=warn) is False:
