@@ -270,7 +270,7 @@ class SymmetricLogUniform(Prior):
 
     def __init__(self, minimum, maximum, name=None, latex_label=None,
                  unit=None, boundary=None):
-        """Symmetric Log-Uniform distribtions with bounds
+        """Symmetric Log-Uniform distributions with bounds
 
         This is identical to a Log-Uniform distribution, but mirrored about
         the zero-axis and subsequently normalized. As such, the distribution
@@ -1001,7 +1001,7 @@ class Beta(Prior):
                 return _ln_prob
             return -np.inf
         else:
-            _ln_prob_sub = -np.inf * np.ones(val.size)
+            _ln_prob_sub = np.full_like(val, -np.inf)
             idx = np.isfinite(_ln_prob) & (val >= self.minimum) & (val <= self.maximum)
             _ln_prob_sub[idx] = _ln_prob[idx]
             return _ln_prob_sub
@@ -1430,8 +1430,8 @@ class Categorical(Prior):
                  unit=None, boundary="periodic"):
         """ An equal-weighted Categorical prior
 
-        Parameters:
-        -----------
+        Parameters
+        ==========
         ncategories: int
             The number of available categories. The prior mass support is then
             integers [0, ncategories - 1].
