@@ -1,7 +1,8 @@
+import datetime
 import inspect
 import json
 import os
-from collections import OrderedDict, namedtuple
+from collections import namedtuple
 from copy import copy
 from importlib import import_module
 from itertools import product
@@ -25,6 +26,9 @@ from .utils import (
 from .prior import Prior, PriorDict, DeltaFunction, ConditionalDeltaFunction
 
 
+EXTENSIONS = ["json", "hdf5", "h5", "pickle", "pkl"]
+
+
 def result_file_name(outdir, label, extension='json', gzip=False):
     """ Returns the standard filename used for a result file
 
@@ -294,7 +298,7 @@ def reweight(result, label=None, new_likelihood=None, new_prior=None,
 
     if conversion_function is not None:
         data_frame = result.posterior
-        if "npool" in inspect.getargspec(conversion_function).args:
+        if "npool" in inspect.signature(conversion_function).parameters:
             data_frame = conversion_function(data_frame, new_likelihood, new_prior, npool=npool)
         else:
             data_frame = conversion_function(data_frame, new_likelihood, new_prior)
@@ -359,7 +363,7 @@ class Result(object):
             The number of times the likelihood function is called
         log_prior_evaluations: array_like
             The evaluations of the prior for each sample point
-        sampling_time: float
+        sampling_time: datetime.timedelta, float
             The time taken to complete the sampling
         nburn: int
             The number of burn-in steps discarded for MCMC samplers
@@ -377,7 +381,7 @@ class Result(object):
             this information is generated when the result object is initialized
 
         Notes
-        =========
+        =====
         All sampling output parameters, e.g. the samples themselves are
         typically not given at initialisation, but set at a later stage.
 
@@ -409,6 +413,8 @@ class Result(object):
         self.log_likelihood_evaluations = log_likelihood_evaluations
         self.log_prior_evaluations = log_prior_evaluations
         self.num_likelihood_evaluations = num_likelihood_evaluations
+        if isinstance(sampling_time, float):
+            sampling_time = datetime.timedelta(seconds=sampling_time)
         self.sampling_time = sampling_time
         self.version = version
         self.max_autocorrelation_time = max_autocorrelation_time
@@ -416,65 +422,6 @@ class Result(object):
         self.prior_values = None
         self._kde = None
 
-    @classmethod
-    def _from_hdf5_old(cls, filename=None, outdir=None, label=None):
-        """ Read in a saved .h5 data file in the old format.
-
-        Parameters
-        ==========
-        filename: str
-            If given, try to load from this filename
-        outdir, label: str
-            If given, use the default naming convention for saved results file
-
-        Returns
-        =======
-        result: bilby.core.result.Result
-
-        Raises
-        =======
-        ValueError: If no filename is given and either outdir or label is None
-                    If no bilby.core.result.Result is found in the path
-
-        """
-        import deepdish
-        filename = _determine_file_name(filename, outdir, label, 'hdf5', False)
-
-        if os.path.isfile(filename):
-            dictionary = deepdish.io.load(filename)
-            # Some versions of deepdish/pytables return the dictionary as
-            # a dictionary with a key 'data'
-            if len(dictionary) == 1 and 'data' in dictionary:
-                dictionary = dictionary['data']
-
-            if "priors" in dictionary:
-                # parse priors from JSON string (allowing for backwards
-                # compatibility)
-                if not isinstance(dictionary["priors"], PriorDict):
-                    try:
-                        priordict = PriorDict()
-                        for key, value in dictionary["priors"].items():
-                            if key not in ["__module__", "__name__", "__prior_dict__"]:
-                                try:
-                                    priordict[key] = decode_bilby_json(value)
-                                except AttributeError:
-                                    continue
-                        dictionary["priors"] = priordict
-                    except Exception as e:
-                        raise IOError(
-                            "Unable to parse priors from '{}':\n{}".format(
-                                filename, e,
-                            )
-                        )
-            try:
-                if isinstance(dictionary.get('posterior', None), dict):
-                    dictionary['posterior'] = pd.DataFrame(dictionary['posterior'])
-                return cls(**dictionary)
-            except TypeError as e:
-                raise IOError("Unable to load dictionary, error={}".format(e))
-        else:
-            raise IOError("No result '{}' found".format(filename))
-
     _load_doctstring = """ Read in a saved .{format} data file
 
     Parameters
@@ -489,7 +436,7 @@ class Result(object):
     result: bilby.core.result.Result
 
     Raises
-    =======
+    ======
     ValueError: If no filename is given and either outdir or label is None
                 If no bilby.core.result.Result is found in the path
 
@@ -510,8 +457,6 @@ class Result(object):
         filename = _determine_file_name(filename, outdir, label, 'hdf5', False)
         with h5py.File(filename, "r") as ff:
             data = recursively_load_dict_contents_from_group(ff, '/')
-        if list(data.keys()) == ["data"]:
-            return cls._from_hdf5_old(filename=filename)
         data["posterior"] = pd.DataFrame(data["posterior"])
         data["priors"] = PriorDict._get_from_json_dict(
             json.loads(data["priors"], object_hook=decode_bilby_json)
@@ -532,10 +477,17 @@ class Result(object):
     @classmethod
     @docstring(_load_doctstring.format(format="json"))
     def from_json(cls, filename=None, outdir=None, label=None, gzip=False):
+        from json.decoder import JSONDecodeError
+
         filename = _determine_file_name(filename, outdir, label, 'json', gzip)
 
         if os.path.isfile(filename):
-            dictionary = load_json(filename, gzip)
+            try:
+                dictionary = load_json(filename, gzip)
+            except JSONDecodeError as e:
+                raise IOError(
+                    "JSON failed to decode {} with message {}".format(filename, e)
+                )
             try:
                 return cls(**dictionary)
             except TypeError as e:
@@ -711,7 +663,7 @@ class Result(object):
             'num_likelihood_evaluations', 'samples', 'nested_samples',
             'walkers', 'nburn', 'parameter_labels', 'parameter_labels_with_unit',
             'version']
-        dictionary = OrderedDict()
+        dictionary = dict()
         for attr in save_attrs:
             try:
                 dictionary[attr] = getattr(self, attr)
@@ -801,7 +753,7 @@ class Result(object):
     def save_posterior_samples(self, filename=None, outdir=None, label=None):
         """ Saves posterior samples to a file
 
-        Generates a .dat file containing the posterior samples and auxillary
+        Generates a .dat file containing the posterior samples and auxiliary
         data saved in the posterior. Note, strings in the posterior are
         removed while complex numbers will be given as absolute values with
         abs appended to the column name
@@ -1145,7 +1097,7 @@ class Result(object):
             to override the outdir set by the absolute path of the result object.
 
         Notes
-        -----
+        =====
             The generation of the corner plot themselves is done by the corner
             python module, see https://corner.readthedocs.io for more
             information.
@@ -1384,7 +1336,7 @@ class Result(object):
             ax.set_ylabel(ylabel)
 
         handles, labels = plt.gca().get_legend_handles_labels()
-        by_label = OrderedDict(zip(labels, handles))
+        by_label = dict(zip(labels, handles))
         plt.legend(by_label.values(), by_label.keys())
         ax.legend(numpoints=3)
         fig.tight_layout()
@@ -1423,22 +1375,21 @@ class Result(object):
             Function which adds in extra parameters to the data frame,
             should take the data_frame, likelihood and prior as arguments.
         """
-        try:
-            data_frame = self.posterior
-        except ValueError:
-            data_frame = pd.DataFrame(
-                self.samples, columns=self.search_parameter_keys)
-            data_frame = self._add_prior_fixed_values_to_posterior(
-                data_frame, priors)
-            data_frame['log_likelihood'] = getattr(
-                self, 'log_likelihood_evaluations', np.nan)
-            if self.log_prior_evaluations is None and priors is not None:
-                data_frame['log_prior'] = priors.ln_prob(
-                    dict(data_frame[self.search_parameter_keys]), axis=0)
-            else:
-                data_frame['log_prior'] = self.log_prior_evaluations
+
+        data_frame = pd.DataFrame(
+            self.samples, columns=self.search_parameter_keys)
+        data_frame = self._add_prior_fixed_values_to_posterior(
+            data_frame, priors)
+        data_frame['log_likelihood'] = getattr(
+            self, 'log_likelihood_evaluations', np.nan)
+        if self.log_prior_evaluations is None and priors is not None:
+            data_frame['log_prior'] = priors.ln_prob(
+                dict(data_frame[self.search_parameter_keys]), axis=0)
+        else:
+            data_frame['log_prior'] = self.log_prior_evaluations
+
         if conversion_function is not None:
-            if "npool" in inspect.getargspec(conversion_function).args:
+            if "npool" in inspect.signature(conversion_function).parameters:
                 data_frame = conversion_function(data_frame, likelihood, priors, npool=npool)
             else:
                 data_frame = conversion_function(data_frame, likelihood, priors)
@@ -1484,8 +1435,11 @@ class Result(object):
         if keys is None:
             keys = self.search_parameter_keys
         if self.injection_parameters is None:
-            raise(TypeError, "Result object has no 'injection_parameters'. "
-                             "Cannot compute credible levels.")
+            raise (
+                TypeError,
+                "Result object has no 'injection_parameters'. "
+                "Cannot compute credible levels."
+            )
         credible_levels = {key: self.get_injection_credible_level(key, weights=weights)
                            for key in keys
                            if isinstance(self.injection_parameters.get(key, None), float)}
@@ -1511,8 +1465,11 @@ class Result(object):
         float: credible level
         """
         if self.injection_parameters is None:
-            raise(TypeError, "Result object has no 'injection_parameters'. "
-                             "Cannot copmute credible levels.")
+            raise (
+                TypeError,
+                "Result object has no 'injection_parameters'. "
+                "Cannot copmute credible levels."
+            )
 
         if weights is None:
             weights = np.ones(len(self.posterior))
@@ -1724,7 +1681,7 @@ class ResultList(list):
     def __init__(self, results=None):
         """ A class to store a list of :class:`bilby.core.result.Result` objects
         from equivalent runs on the same data. This provides methods for
-        outputing combined results.
+        outputting combined results.
 
         Parameters
         ==========
@@ -1820,7 +1777,7 @@ class ResultList(list):
         result.log_evidence = logsumexp(log_evidences, b=1. / len(self))
         result.log_bayes_factor = result.log_evidence - result.log_noise_evidence
 
-        # Propogate uncertainty in combined evidence
+        # Propagate uncertainty in combined evidence
         log_errs = [res.log_evidence_err for res in self if np.isfinite(res.log_evidence_err)]
         if len(log_errs) > 0:
             result.log_evidence_err = 0.5 * logsumexp(2 * np.array(log_errs), b=1. / len(self))
@@ -1866,7 +1823,7 @@ class ResultList(list):
         result.log_evidence = logsumexp(log_evidences, b=1. / len(self))
         result.log_bayes_factor = result.log_evidence - result.log_noise_evidence
 
-        # Propogate uncertainty in combined evidence
+        # Propagate uncertainty in combined evidence
         log_errs = [res.log_evidence_err for res in self if np.isfinite(res.log_evidence_err)]
         if len(log_errs) > 0:
             result.log_evidence_err = 0.5 * logsumexp(2 * np.array(log_errs), b=1. / len(self))
@@ -1898,7 +1855,7 @@ class ResultList(list):
             raise ResultListError("Inconsistent parameters between results")
 
     def check_consistent_data(self):
-        if not np.all([res.log_noise_evidence == self[0].log_noise_evidence for res in self])\
+        if not np.allclose([res.log_noise_evidence for res in self], self[0].log_noise_evidence, atol=1e-8, rtol=0.0)\
                 and not np.all([np.isnan(res.log_noise_evidence) for res in self]):
             raise ResultListError("Inconsistent data between results")
 
@@ -1982,12 +1939,17 @@ def plot_multiple(results, filename=None, labels=None, colours=None,
 
     if evidences:
         if np.isnan(results[0].log_bayes_factor):
-            template = ' $\mathrm{{ln}}(Z)={lnz:1.3g}$'
+            template = r'{label} $\mathrm{{ln}}(Z)={lnz:1.3g}$'
         else:
-            template = ' $\mathrm{{ln}}(B)={lnbf:1.3g}$'
-        labels = [template.format(lnz=result.log_evidence,
-                                  lnbf=result.log_bayes_factor)
-                  for ii, result in enumerate(results)]
+            template = r'{label} $\mathrm{{ln}}(B)={lnbf:1.3g}$'
+        labels = [
+            template.format(
+                label=label,
+                lnz=result.log_evidence,
+                lnbf=result.log_bayes_factor,
+            )
+            for label, result in zip(labels, results)
+        ]
 
     axes = fig.get_axes()
     ndim = int(np.sqrt(len(axes)))
@@ -2026,6 +1988,8 @@ def make_pp_plot(results, filename=None, save=True, confidence_interval=[0.68, 0
         The font size for the legend
     keys: list
         A list of keys to use, if None defaults to search_parameter_keys
+    title: bool
+        Whether to add the number of results and total p-value as a plot title
     confidence_interval_alpha: float, list, optional
         The transparency for the background condifence interval
     weight_list: list, optional
@@ -2047,11 +2011,12 @@ def make_pp_plot(results, filename=None, save=True, confidence_interval=[0.68, 0
     if weight_list is None:
         weight_list = [None] * len(results)
 
-    credible_levels = pd.DataFrame()
+    credible_levels = list()
     for i, result in enumerate(results):
-        credible_levels = credible_levels.append(
-            result.get_all_injection_credible_levels(keys, weights=weight_list[i]),
-            ignore_index=True)
+        credible_levels.append(
+            result.get_all_injection_credible_levels(keys, weights=weight_list[i])
+        )
+    credible_levels = pd.DataFrame(credible_levels)
 
     if lines is None:
         colors = ["C{}".format(i) for i in range(8)]
@@ -2136,7 +2101,7 @@ class ResultError(Exception):
 
 
 class ResultListError(ResultError):
-    """ For Errors occuring during combining results. """
+    """ For Errors occurring during combining results. """
 
 
 class FileMovedError(ResultError):
