@@ -1,8 +1,7 @@
-
 import numpy as np
 from pandas import DataFrame
 
-from .base_sampler import NestedSampler
+from .base_sampler import NestedSampler, signal_wrapper
 
 
 class Nestle(NestedSampler):
@@ -25,30 +24,45 @@ class Nestle(NestedSampler):
         sampling
 
     """
-    default_kwargs = dict(verbose=True, method='multi', npoints=500,
-                          update_interval=None, npdim=None, maxiter=None,
-                          maxcall=None, dlogz=None, decline_factor=None,
-                          rstate=None, callback=None, steps=20, enlarge=1.2)
+
+    default_kwargs = dict(
+        verbose=True,
+        method="multi",
+        npoints=500,
+        update_interval=None,
+        npdim=None,
+        maxiter=None,
+        maxcall=None,
+        dlogz=None,
+        decline_factor=None,
+        rstate=None,
+        callback=None,
+        steps=20,
+        enlarge=1.2,
+    )
 
     def _translate_kwargs(self, kwargs):
-        if 'npoints' not in kwargs:
+        kwargs = super()._translate_kwargs(kwargs)
+        if "npoints" not in kwargs:
             for equiv in self.npoints_equiv_kwargs:
                 if equiv in kwargs:
-                    kwargs['npoints'] = kwargs.pop(equiv)
-        if 'steps' not in kwargs:
+                    kwargs["npoints"] = kwargs.pop(equiv)
+        if "steps" not in kwargs:
             for equiv in self.walks_equiv_kwargs:
                 if equiv in kwargs:
-                    kwargs['steps'] = kwargs.pop(equiv)
+                    kwargs["steps"] = kwargs.pop(equiv)
 
     def _verify_kwargs_against_default_kwargs(self):
-        if self.kwargs['verbose']:
+        if self.kwargs["verbose"]:
             import nestle
-            self.kwargs['callback'] = nestle.print_progress
-            self.kwargs.pop('verbose')
+
+            self.kwargs["callback"] = nestle.print_progress
+            self.kwargs.pop("verbose")
         NestedSampler._verify_kwargs_against_default_kwargs(self)
 
+    @signal_wrapper
     def run_sampler(self):
-        """ Runs Nestle sampler with given kwargs and returns the result
+        """Runs Nestle sampler with given kwargs and returns the result
 
         Returns
         =======
@@ -56,21 +70,27 @@ class Nestle(NestedSampler):
 
         """
         import nestle
+
         out = nestle.sample(
             loglikelihood=self.log_likelihood,
             prior_transform=self.prior_transform,
-            ndim=self.ndim, **self.kwargs)
+            ndim=self.ndim,
+            **self.kwargs
+        )
         print("")
 
         self.result.sampler_output = out
         self.result.samples = nestle.resample_equal(out.samples, out.weights)
         self.result.nested_samples = DataFrame(
-            out.samples, columns=self.search_parameter_keys)
-        self.result.nested_samples['weights'] = out.weights
-        self.result.nested_samples['log_likelihood'] = out.logl
+            out.samples, columns=self.search_parameter_keys
+        )
+        self.result.nested_samples["weights"] = out.weights
+        self.result.nested_samples["log_likelihood"] = out.logl
         self.result.log_likelihood_evaluations = self.reorder_loglikelihoods(
-            unsorted_loglikelihoods=out.logl, unsorted_samples=out.samples,
-            sorted_samples=self.result.samples)
+            unsorted_loglikelihoods=out.logl,
+            unsorted_samples=out.samples,
+            sorted_samples=self.result.samples,
+        )
         self.result.log_evidence = out.logz
         self.result.log_evidence_err = out.logzerr
         self.result.information_gain = out.h
@@ -88,14 +108,24 @@ class Nestle(NestedSampler):
 
         """
         import nestle
+
         kwargs = self.kwargs.copy()
-        kwargs['maxiter'] = 2
+        kwargs["maxiter"] = 2
         nestle.sample(
             loglikelihood=self.log_likelihood,
             prior_transform=self.prior_transform,
-            ndim=self.ndim, **kwargs)
+            ndim=self.ndim,
+            **kwargs
+        )
         self.result.samples = np.random.uniform(0, 1, (100, self.ndim))
         self.result.log_evidence = np.nan
         self.result.log_evidence_err = np.nan
         self.calc_likelihood_count()
         return self.result
+
+    def write_current_state(self):
+        """
+        Nestle doesn't support checkpointing so no current state will be
+        written on interrupt.
+        """
+        pass
