@@ -1,6 +1,6 @@
 from . import (conversion, cosmology, detector, eos, likelihood, prior,
                result, source, utils, waveform_generator)
-from .waveform_generator import WaveformGenerator
+from .waveform_generator import WaveformGenerator, LALCBCWaveformGenerator
 from .likelihood import GravitationalWaveTransient
 from .detector import calibration
 
