@@ -1,15 +1,22 @@
+"""
+A collection of functions to convert between parameters describing
+gravitational-wave sources.
+"""
+
+import os
 import sys
 import multiprocessing
+import pickle
 
 import numpy as np
-from pandas import DataFrame
+from pandas import DataFrame, Series
 
 from ..core.likelihood import MarginalizedLikelihoodReconstructionError
-from ..core.utils import logger, solar_mass
+from ..core.utils import logger, solar_mass, command_line_args
 from ..core.prior import DeltaFunction
 from .utils import lalsim_SimInspiralTransformPrecessingNewInitialConditions
 from .eos.eos import SpectralDecompositionEOS, EOSFamily, IntegrateTOV
-from .cosmology import get_cosmology
+from .cosmology import get_cosmology, z_at_value
 
 
 def redshift_to_luminosity_distance(redshift, cosmology=None):
@@ -22,19 +29,19 @@ def redshift_to_comoving_distance(redshift, cosmology=None):
     return cosmology.comoving_distance(redshift).value
 
 
-@np.vectorize
 def luminosity_distance_to_redshift(distance, cosmology=None):
     from astropy import units
-    from astropy.cosmology import z_at_value
     cosmology = get_cosmology(cosmology)
+    if isinstance(distance, Series):
+        distance = distance.values
     return z_at_value(cosmology.luminosity_distance, distance * units.Mpc)
 
 
-@np.vectorize
 def comoving_distance_to_redshift(distance, cosmology=None):
     from astropy import units
-    from astropy.cosmology import z_at_value
     cosmology = get_cosmology(cosmology)
+    if isinstance(distance, Series):
+        distance = distance.values
     return z_at_value(cosmology.comoving_distance, distance * units.Mpc)
 
 
@@ -50,33 +57,44 @@ def luminosity_distance_to_comoving_distance(distance, cosmology=None):
     return redshift_to_comoving_distance(redshift, cosmology)
 
 
+_cosmology_docstring = """
+Convert from {input} to {output}
+
+Parameters
+----------
+{input}: float
+    The {input} to convert.
+cosmology: astropy.cosmology.Cosmology
+    The cosmology to use for the transformation.
+    See :code:`bilby.gw.cosmology.get_cosmology` for details of how to
+    specify this.
+
+Returns
+-------
+float
+    The {output} corresponding to the provided {input}.
+"""
+
+for _func in [
+    comoving_distance_to_luminosity_distance,
+    comoving_distance_to_redshift,
+    luminosity_distance_to_comoving_distance,
+    luminosity_distance_to_redshift,
+    redshift_to_comoving_distance,
+    redshift_to_luminosity_distance,
+]:
+    input, output = _func.__name__.split("_to_")
+    _func.__doc__ = _cosmology_docstring.format(input=input, output=output)
+
+
 def bilby_to_lalsimulation_spins(
-        theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1, mass_2,
-        reference_frequency, phase):
-    if (a_1 == 0 or tilt_1 in [0, np.pi]) and (a_2 == 0 or tilt_2 in [0, np.pi]):
-        spin_1x = 0
-        spin_1y = 0
-        spin_1z = a_1 * np.cos(tilt_1)
-        spin_2x = 0
-        spin_2y = 0
-        spin_2z = a_2 * np.cos(tilt_2)
-        iota = theta_jn
-    else:
-        iota, spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z = \
-            transform_precessing_spins(
-                theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1,
-                mass_2, reference_frequency, phase)
-    return iota, spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z
-
-
-@np.vectorize
-def transform_precessing_spins(theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1,
-                               a_2, mass_1, mass_2, reference_frequency, phase):
+    theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1, mass_2,
+    reference_frequency, phase
+):
     """
-    Vectorized version of
-    lalsimulation.SimInspiralTransformPrecessingNewInitialConditions
+    Convert from Bilby spin parameters to lalsimulation ones.
 
-    All parameters are defined at the reference frequency
+    All parameters are defined at the reference frequency and in SI units.
 
     Parameters
     ==========
@@ -95,9 +113,9 @@ def transform_precessing_spins(theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1,
     a_2: float
         Secondary dimensionless spin magnitude
     mass_1: float
-        Primary mass _in SI units_
+        Primary mass in SI units
     mass_2: float
-        Secondary mass _in SI units_
+        Secondary mass in SI units
     reference_frequency: float
     phase: float
         Orbital phase
@@ -109,15 +127,42 @@ def transform_precessing_spins(theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1,
     spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z: float
         Cartesian spin components
     """
-
-    iota, spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z = (
-        lalsim_SimInspiralTransformPrecessingNewInitialConditions(
-            theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1, mass_2,
-            reference_frequency, phase))
-
+    if (a_1 == 0 or tilt_1 in [0, np.pi]) and (a_2 == 0 or tilt_2 in [0, np.pi]):
+        spin_1x = 0
+        spin_1y = 0
+        spin_1z = a_1 * np.cos(tilt_1)
+        spin_2x = 0
+        spin_2y = 0
+        spin_2z = a_2 * np.cos(tilt_2)
+        iota = theta_jn
+    else:
+        from numbers import Number
+        args = (
+            theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1,
+            mass_2, reference_frequency, phase
+        )
+        float_inputs = all([isinstance(arg, Number) for arg in args])
+        if float_inputs:
+            func = lalsim_SimInspiralTransformPrecessingNewInitialConditions
+        else:
+            func = transform_precessing_spins
+        iota, spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z = func(*args)
     return iota, spin_1x, spin_1y, spin_1z, spin_2x, spin_2y, spin_2z
 
 
+@np.vectorize
+def transform_precessing_spins(*args):
+    """
+    Vectorized wrapper for
+    lalsimulation.SimInspiralTransformPrecessingNewInitialConditions
+
+    For detailed documentation see
+    :code:`bilby.gw.conversion.bilby_to_lalsimulation_spins`.
+    This will be removed from the public API in a future release.
+    """
+    return lalsim_SimInspiralTransformPrecessingNewInitialConditions(*args)
+
+
 def convert_to_lal_binary_black_hole_parameters(parameters):
     """
     Convert parameters we have into parameters we need.
@@ -165,67 +210,8 @@ def convert_to_lal_binary_black_hole_parameters(parameters):
             converted_parameters[key[:-7]] = converted_parameters[key] * (
                 1 + converted_parameters['redshift'])
 
-    if 'chirp_mass' in converted_parameters.keys():
-        if "mass_1" in converted_parameters.keys():
-            converted_parameters["mass_ratio"] = chirp_mass_and_primary_mass_to_mass_ratio(
-                converted_parameters["chirp_mass"], converted_parameters["mass_1"])
-        if 'total_mass' in converted_parameters.keys():
-            converted_parameters['symmetric_mass_ratio'] =\
-                chirp_mass_and_total_mass_to_symmetric_mass_ratio(
-                    converted_parameters['chirp_mass'],
-                    converted_parameters['total_mass'])
-        if 'symmetric_mass_ratio' in converted_parameters.keys():
-            converted_parameters['mass_ratio'] =\
-                symmetric_mass_ratio_to_mass_ratio(
-                    converted_parameters['symmetric_mass_ratio'])
-        if 'total_mass' not in converted_parameters.keys():
-            converted_parameters['total_mass'] =\
-                chirp_mass_and_mass_ratio_to_total_mass(
-                    converted_parameters['chirp_mass'],
-                    converted_parameters['mass_ratio'])
-        converted_parameters['mass_1'], converted_parameters['mass_2'] = \
-            total_mass_and_mass_ratio_to_component_masses(
-                converted_parameters['mass_ratio'],
-                converted_parameters['total_mass'])
-    elif 'total_mass' in converted_parameters.keys():
-        if 'symmetric_mass_ratio' in converted_parameters.keys():
-            converted_parameters['mass_ratio'] = \
-                symmetric_mass_ratio_to_mass_ratio(
-                    converted_parameters['symmetric_mass_ratio'])
-        if 'mass_ratio' in converted_parameters.keys():
-            converted_parameters['mass_1'], converted_parameters['mass_2'] =\
-                total_mass_and_mass_ratio_to_component_masses(
-                    converted_parameters['mass_ratio'],
-                    converted_parameters['total_mass'])
-        elif 'mass_1' in converted_parameters.keys():
-            converted_parameters['mass_2'] =\
-                converted_parameters['total_mass'] -\
-                converted_parameters['mass_1']
-        elif 'mass_2' in converted_parameters.keys():
-            converted_parameters['mass_1'] = \
-                converted_parameters['total_mass'] - \
-                converted_parameters['mass_2']
-    elif 'symmetric_mass_ratio' in converted_parameters.keys():
-        converted_parameters['mass_ratio'] =\
-            symmetric_mass_ratio_to_mass_ratio(
-                converted_parameters['symmetric_mass_ratio'])
-        if 'mass_1' in converted_parameters.keys():
-            converted_parameters['mass_2'] =\
-                converted_parameters['mass_1'] *\
-                converted_parameters['mass_ratio']
-        elif 'mass_2' in converted_parameters.keys():
-            converted_parameters['mass_1'] =\
-                converted_parameters['mass_2'] /\
-                converted_parameters['mass_ratio']
-    elif 'mass_ratio' in converted_parameters.keys():
-        if 'mass_1' in converted_parameters.keys():
-            converted_parameters['mass_2'] =\
-                converted_parameters['mass_1'] *\
-                converted_parameters['mass_ratio']
-        if 'mass_2' in converted_parameters.keys():
-            converted_parameters['mass_1'] = \
-                converted_parameters['mass_2'] /\
-                converted_parameters['mass_ratio']
+    # we do not require the component masses be added if no mass parameters are present
+    converted_parameters = generate_component_masses(converted_parameters, require_add=False)
 
     for idx in ['1', '2']:
         key = 'chi_{}'.format(idx)
@@ -244,19 +230,28 @@ def convert_to_lal_binary_black_hole_parameters(parameters):
                     converted_parameters[key])
                 converted_parameters['cos_tilt_{}'.format(idx)] = \
                     np.sign(converted_parameters[key])
-                converted_parameters['phi_jl'] = 0.0
-                converted_parameters['phi_12'] = 0.0
             else:
-                converted_parameters[f"cos_tilt_{idx}"] = (
-                    converted_parameters[key] / converted_parameters[f"a_{idx}"]
-                )
+                with np.errstate(invalid="raise"):
+                    try:
+                        converted_parameters[f"cos_tilt_{idx}"] = (
+                            converted_parameters[key] / converted_parameters[f"a_{idx}"]
+                        )
+                    except (FloatingPointError, ZeroDivisionError):
+                        logger.debug(
+                            "Error in conversion to spherical spin tilt. "
+                            "This is often due to the spin parameters being zero. "
+                            f"Setting cos_tilt_{idx} = 1."
+                        )
+                        converted_parameters[f"cos_tilt_{idx}"] = 1.0
+
+    for key in ["phi_jl", "phi_12"]:
+        if key not in converted_parameters:
+            converted_parameters[key] = 0.0
 
     for angle in ['tilt_1', 'tilt_2', 'theta_jn']:
         cos_angle = str('cos_' + angle)
         if cos_angle in converted_parameters.keys():
-            with np.errstate(invalid="ignore"):
-                converted_parameters[angle] =\
-                    np.arccos(converted_parameters[cos_angle])
+            converted_parameters[angle] = np.arccos(converted_parameters[cos_angle])
 
     if "delta_phase" in original_keys:
         with np.errstate(invalid="ignore"):
@@ -426,6 +421,33 @@ def total_mass_and_mass_ratio_to_component_masses(mass_ratio, total_mass):
     return mass_1, mass_2
 
 
+def chirp_mass_and_mass_ratio_to_component_masses(chirp_mass, mass_ratio):
+    """
+    Convert total mass and mass ratio of a binary to its component masses.
+
+    Parameters
+    ==========
+    chirp_mass: float
+        Chirp mass of the binary
+    mass_ratio: float
+        Mass ratio (mass_2/mass_1) of the binary
+
+    Returns
+    =======
+    mass_1: float
+        Mass of the heavier object
+    mass_2: float
+        Mass of the lighter object
+    """
+    total_mass = chirp_mass_and_mass_ratio_to_total_mass(chirp_mass=chirp_mass,
+                                                         mass_ratio=mass_ratio)
+    mass_1, mass_2 = (
+        total_mass_and_mass_ratio_to_component_masses(
+            total_mass=total_mass, mass_ratio=mass_ratio)
+    )
+    return mass_1, mass_2
+
+
 def symmetric_mass_ratio_to_mass_ratio(symmetric_mass_ratio):
     """
     Convert the symmetric mass ratio to the normal mass ratio.
@@ -474,7 +496,7 @@ def chirp_mass_and_primary_mass_to_mass_ratio(chirp_mass, mass_1):
 
         (chirp_mass/mass_1)^5 = q^3 / (1 + q)
 
-    Solving for q, we find the releation expressed in python below for q.
+    Solving for q, we find the relation expressed in python below for q.
 
     Parameters
     ==========
@@ -624,6 +646,30 @@ def mass_1_and_chirp_mass_to_mass_ratio(mass_1, chirp_mass):
     return mass_ratio
 
 
+def mass_2_and_chirp_mass_to_mass_ratio(mass_2, chirp_mass):
+    """
+    Calculate mass ratio from mass_1 and chirp_mass.
+
+    This involves solving mc = m2 * (1/q)**(3/5) / (1 + (1/q))**(1/5).
+
+    Parameters
+    ==========
+    mass_2: float
+        Mass of the lighter object
+    chirp_mass: float
+        Chirp mass of the binary
+
+    Returns
+    =======
+    mass_ratio: float
+        Mass ratio of the binary
+    """
+    # Passing mass_2, the expression from the function above
+    # returns 1/q (because chirp mass is invariant under
+    # mass_1 <-> mass_2)
+    return 1 / mass_1_and_chirp_mass_to_mass_ratio(mass_2, chirp_mass)
+
+
 def lambda_1_lambda_2_to_lambda_tilde(lambda_1, lambda_2, mass_1, mass_2):
     """
     Convert from individual tidal parameters to domainant tidal term.
@@ -642,7 +688,7 @@ def lambda_1_lambda_2_to_lambda_tilde(lambda_1, lambda_2, mass_1, mass_2):
         Mass of less massive neutron star.
 
     Returns
-    ======
+    =======
     lambda_tilde: float
         Dominant tidal term.
     """
@@ -784,12 +830,8 @@ def _generate_all_cbc_parameters(sample, defaults, base_conversion,
     output_sample = fill_from_fixed_priors(output_sample, priors)
     output_sample, _ = base_conversion(output_sample)
     if likelihood is not None:
-        if (
-                hasattr(likelihood, 'phase_marginalization') or
-                hasattr(likelihood, 'time_marginalization') or
-                hasattr(likelihood, 'distance_marginalization') or
-                hasattr(likelihood, 'calibration_marginalization')
-        ):
+        marginalized_parameters = getattr(likelihood, "_marginalized_parameters", list())
+        if len(marginalized_parameters) > 0:
             try:
                 generate_posterior_samples_from_marginalized_likelihood(
                     samples=output_sample, likelihood=likelihood, npool=npool)
@@ -800,10 +842,17 @@ def _generate_all_cbc_parameters(sample, defaults, base_conversion,
                     "interpretation.".format(e)
                 )
         if priors is not None:
-            for par, name in zip(
-                    ['distance', 'phase', 'time'],
-                    ['luminosity_distance', 'phase', 'geocent_time']):
-                if getattr(likelihood, '{}_marginalization'.format(par), False):
+            misnamed_marginalizations = dict(
+                distance="luminosity_distance",
+                time="geocent_time",
+                calibration="recalib_index",
+            )
+            for par in marginalized_parameters:
+                name = misnamed_marginalizations.get(par, par)
+                if (
+                    getattr(likelihood, f'{par}_marginalization', False)
+                    and name in likelihood.priors
+                ):
                     priors[name] = likelihood.priors[name]
 
         if (
@@ -819,6 +868,8 @@ def _generate_all_cbc_parameters(sample, defaults, base_conversion,
                     "Failed to generate sky frame parameters for type {}"
                     .format(type(output_sample))
                 )
+    if likelihood is not None:
+        compute_snrs(output_sample, likelihood, npool=npool)
     for key, func in zip(["mass", "spin", "source frame"], [
             generate_mass_parameters, generate_spin_parameters,
             generate_source_frame_parameters]):
@@ -828,8 +879,6 @@ def _generate_all_cbc_parameters(sample, defaults, base_conversion,
             logger.info(
                 "Generation of {} parameters failed with message {}".format(
                     key, e))
-    if likelihood is not None:
-        compute_snrs(output_sample, likelihood)
     return output_sample
 
 
@@ -897,6 +946,29 @@ def generate_all_bns_parameters(sample, likelihood=None, priors=None, npool=1):
 
 
 def generate_specific_parameters(sample, parameters):
+    """
+    Generate a specific subset of parameters that can be generated.
+
+    Parameters
+    ----------
+    sample: dict
+        The input sample to be converted.
+    parameters: list
+        The list of parameters to return.
+
+    Returns
+    -------
+    output_sample: dict
+        The converted parameters
+
+    Notes
+    -----
+    This is _not_ an optimized function. Under the hood, it generates all
+    possible parameters and then downselects.
+
+    If the passed :code:`parameters` do not include the input parameters,
+    those will not be returned.
+    """
     updated_sample = generate_all_bns_parameters(sample=sample.copy())
     output_sample = sample.__class__()
     for key in parameters:
@@ -929,33 +1001,220 @@ def fill_from_fixed_priors(sample, priors):
     return output_sample
 
 
-def generate_mass_parameters(sample):
-    """
-    Add the known mass parameters to the data frame/dictionary.
-
+def generate_component_masses(sample, require_add=False, source=False):
+    """"
+    Add the component masses to the dataframe/dictionary
     We add:
-        chirp mass, total mass, symmetric mass ratio, mass ratio
+        mass_1, mass_2
+    We also add any other masses which may be necessary for
+    intermediate steps, i.e. typically the  total mass is necessary, along
+    with the mass ratio, so these will usually be added to the dictionary
+
+    If `require_add` is True, then having an incomplete set of mass
+    parameters (so that the component mass parameters cannot be added)
+    will throw an error, otherwise it will quietly add nothing to the
+    dictionary.
+
+    Parameters
+    =========
+    sample : dict
+        The input dictionary with at least one
+        component with overall mass scaling (i.e.
+        chirp_mass, mass_1, mass_2, total_mass) and
+        then any other mass parameter.
+    source : bool, default False
+        If True, then perform the conversions for source mass parameters
+        i.e. mass_1_source instead of mass_1
+
+    Returns
+    dict : the updated dictionary
+    """
+    def check_and_return_quietly(require_add, sample):
+        if require_add:
+            raise KeyError("Insufficient mass parameters in input dictionary")
+        else:
+            return sample
+    output_sample = sample.copy()
+
+    if source:
+        mass_1_key = "mass_1_source"
+        mass_2_key = "mass_2_source"
+        total_mass_key = "total_mass_source"
+        chirp_mass_key = "chirp_mass_source"
+    else:
+        mass_1_key = "mass_1"
+        mass_2_key = "mass_2"
+        total_mass_key = "total_mass"
+        chirp_mass_key = "chirp_mass"
+
+    if mass_1_key in sample.keys():
+        if mass_2_key in sample.keys():
+            return output_sample
+        if total_mass_key in sample.keys():
+            output_sample[mass_2_key] = output_sample[total_mass_key] - (
+                output_sample[mass_1_key]
+            )
+            return output_sample
+
+        elif "mass_ratio" in sample.keys():
+            pass
+        elif "symmetric_mass_ratio" in sample.keys():
+            output_sample["mass_ratio"] = (
+                symmetric_mass_ratio_to_mass_ratio(
+                    output_sample["symmetric_mass_ratio"])
+            )
+        elif chirp_mass_key in sample.keys():
+            output_sample["mass_ratio"] = (
+                mass_1_and_chirp_mass_to_mass_ratio(
+                    mass_1=output_sample[mass_1_key],
+                    chirp_mass=output_sample[chirp_mass_key])
+            )
+        else:
+            return check_and_return_quietly(require_add, sample)
+
+        output_sample[mass_2_key] = (
+            output_sample["mass_ratio"] * output_sample[mass_1_key]
+        )
+
+        return output_sample
+
+    elif mass_2_key in sample.keys():
+        # mass_1 is not in the dict
+        if total_mass_key in sample.keys():
+            output_sample[mass_1_key] = (
+                output_sample[total_mass_key] - output_sample[mass_2_key]
+            )
+            return output_sample
+        elif "mass_ratio" in sample.keys():
+            pass
+        elif "symmetric_mass_ratio" in sample.keys():
+            output_sample["mass_ratio"] = (
+                symmetric_mass_ratio_to_mass_ratio(
+                    output_sample["symmetric_mass_ratio"])
+            )
+        elif chirp_mass_key in sample.keys():
+            output_sample["mass_ratio"] = (
+                mass_2_and_chirp_mass_to_mass_ratio(
+                    mass_2=output_sample[mass_2_key],
+                    chirp_mass=output_sample[chirp_mass_key])
+            )
+        else:
+            check_and_return_quietly(require_add, sample)
+
+        output_sample[mass_1_key] = 1 / output_sample["mass_ratio"] * (
+            output_sample[mass_2_key]
+        )
+
+        return output_sample
+
+    # Only if neither mass_1 or mass_2 is in the input sample
+    if total_mass_key in sample.keys():
+        if "mass_ratio" in sample.keys():
+            pass  # We have everything we need already
+        elif "symmetric_mass_ratio" in sample.keys():
+            output_sample["mass_ratio"] = (
+                symmetric_mass_ratio_to_mass_ratio(
+                    output_sample["symmetric_mass_ratio"])
+            )
+        elif chirp_mass_key in sample.keys():
+            output_sample["symmetric_mass_ratio"] = (
+                chirp_mass_and_total_mass_to_symmetric_mass_ratio(
+                    chirp_mass=output_sample[chirp_mass_key],
+                    total_mass=output_sample[total_mass_key])
+            )
+            output_sample["mass_ratio"] = (
+                symmetric_mass_ratio_to_mass_ratio(
+                    output_sample["symmetric_mass_ratio"])
+            )
+        else:
+            return check_and_return_quietly(require_add, sample)
+
+    elif chirp_mass_key in sample.keys():
+        if "mass_ratio" in sample.keys():
+            pass
+        elif "symmetric_mass_ratio" in sample.keys():
+            output_sample["mass_ratio"] = (
+                symmetric_mass_ratio_to_mass_ratio(
+                    sample["symmetric_mass_ratio"])
+            )
+        else:
+            return check_and_return_quietly(require_add, sample)
+
+        output_sample[total_mass_key] = (
+            chirp_mass_and_mass_ratio_to_total_mass(
+                chirp_mass=output_sample[chirp_mass_key],
+                mass_ratio=output_sample["mass_ratio"])
+        )
+
+    # We haven't matched any of the criteria
+    if total_mass_key not in output_sample.keys() or (
+            "mass_ratio" not in output_sample.keys()):
+        return check_and_return_quietly(require_add, sample)
+    mass_1, mass_2 = (
+        total_mass_and_mass_ratio_to_component_masses(
+            total_mass=output_sample[total_mass_key],
+            mass_ratio=output_sample["mass_ratio"])
+    )
+    output_sample[mass_1_key] = mass_1
+    output_sample[mass_2_key] = mass_2
+
+    return output_sample
+
+
+def generate_mass_parameters(sample, source=False):
+    """
+    Add the known mass parameters to the data frame/dictionary.  We do
+    not recompute keys already present in the dictionary
+
+    We add, potentially:
+        chirp mass, total mass, symmetric mass ratio, mass ratio, mass_1, mass_2
 
     Parameters
     ==========
     sample : dict
-        The input dictionary with component masses 'mass_1' and 'mass_2'
-
+        The input dictionary with two "spanning" mass parameters
+        e.g. (mass_1, mass_2), or (chirp_mass, mass_ratio), but not e.g. only
+        (mass_ratio, symmetric_mass_ratio)
     Returns
     =======
     dict: The updated dictionary
 
     """
-    output_sample = sample.copy()
-    output_sample['chirp_mass'] =\
-        component_masses_to_chirp_mass(sample['mass_1'], sample['mass_2'])
-    output_sample['total_mass'] =\
-        component_masses_to_total_mass(sample['mass_1'], sample['mass_2'])
-    output_sample['symmetric_mass_ratio'] =\
-        component_masses_to_symmetric_mass_ratio(sample['mass_1'],
-                                                 sample['mass_2'])
-    output_sample['mass_ratio'] =\
-        component_masses_to_mass_ratio(sample['mass_1'], sample['mass_2'])
+    # Only add the parameters if they're not already present
+    intermediate_sample = generate_component_masses(sample, source=source)
+    output_sample = intermediate_sample.copy()
+
+    if source:
+        mass_1_key = 'mass_1_source'
+        mass_2_key = 'mass_2_source'
+        total_mass_key = 'total_mass_source'
+        chirp_mass_key = 'chirp_mass_source'
+    else:
+        mass_1_key = 'mass_1'
+        mass_2_key = 'mass_2'
+        total_mass_key = 'total_mass'
+        chirp_mass_key = 'chirp_mass'
+
+    if chirp_mass_key not in output_sample.keys():
+        output_sample[chirp_mass_key] = (
+            component_masses_to_chirp_mass(output_sample[mass_1_key],
+                                           output_sample[mass_2_key])
+        )
+    if total_mass_key not in output_sample.keys():
+        output_sample[total_mass_key] = (
+            component_masses_to_total_mass(output_sample[mass_1_key],
+                                           output_sample[mass_2_key])
+        )
+    if 'symmetric_mass_ratio' not in output_sample.keys():
+        output_sample['symmetric_mass_ratio'] = (
+            component_masses_to_symmetric_mass_ratio(output_sample[mass_1_key],
+                                                     output_sample[mass_2_key])
+        )
+    if 'mass_ratio' not in output_sample.keys():
+        output_sample['mass_ratio'] = (
+            component_masses_to_mass_ratio(output_sample[mass_1_key],
+                                           output_sample[mass_2_key])
+        )
 
     return output_sample
 
@@ -1121,7 +1380,7 @@ def generate_source_frame_parameters(sample):
     return output_sample
 
 
-def compute_snrs(sample, likelihood):
+def compute_snrs(sample, likelihood, npool=1):
     """
     Compute the optimal and matched filter snrs of all posterior samples
     and print it out.
@@ -1136,9 +1395,9 @@ def compute_snrs(sample, likelihood):
     """
     if likelihood is not None:
         if isinstance(sample, dict):
+            likelihood.parameters.update(sample)
             signal_polarizations =\
                 likelihood.waveform_generator.frequency_domain_strain(sample)
-            likelihood.parameters.update(sample)
             for ifo in likelihood.interferometers:
                 per_detector_snr = likelihood.calculate_snrs(
                     signal_polarizations, ifo)
@@ -1148,39 +1407,61 @@ def compute_snrs(sample, likelihood):
                     per_detector_snr.optimal_snr_squared.real ** 0.5
         else:
             from tqdm.auto import tqdm
-            logger.info(
-                'Computing SNRs for every sample.')
+            logger.info('Computing SNRs for every sample.')
 
-            matched_filter_snrs = {
-                ifo.name: [] for ifo in likelihood.interferometers}
-            optimal_snrs = {ifo.name: [] for ifo in likelihood.interferometers}
+            fill_args = [(ii, row) for ii, row in sample.iterrows()]
+            if npool > 1:
+                from ..core.sampler.base_sampler import _initialize_global_variables
+                pool = multiprocessing.Pool(
+                    processes=npool,
+                    initializer=_initialize_global_variables,
+                    initargs=(likelihood, None, None, False),
+                )
+                logger.info(
+                    "Using a pool with size {} for nsamples={}".format(npool, len(sample))
+                )
+                new_samples = pool.map(_compute_snrs, tqdm(fill_args, file=sys.stdout))
+                pool.close()
+                pool.join()
+            else:
+                from ..core.sampler.base_sampler import _sampling_convenience_dump
+                _sampling_convenience_dump.likelihood = likelihood
+                new_samples = [_compute_snrs(xx) for xx in tqdm(fill_args, file=sys.stdout)]
 
-            for ii in tqdm(range(len(sample)), file=sys.stdout):
-                signal_polarizations =\
-                    likelihood.waveform_generator.frequency_domain_strain(
-                        dict(sample.iloc[ii]))
-                likelihood.parameters.update(sample.iloc[ii])
-                for ifo in likelihood.interferometers:
-                    per_detector_snr = likelihood.calculate_snrs(
-                        signal_polarizations, ifo)
+            for ii, ifo in enumerate(likelihood.interferometers):
+                matched_filter_snrs = list()
+                optimal_snrs = list()
+                mf_key = '{}_matched_filter_snr'.format(ifo.name)
+                optimal_key = '{}_optimal_snr'.format(ifo.name)
+                for new_sample in new_samples:
+                    matched_filter_snrs.append(new_sample[ii].complex_matched_filter_snr)
+                    optimal_snrs.append(new_sample[ii].optimal_snr_squared.real ** 0.5)
 
-                    matched_filter_snrs[ifo.name].append(
-                        per_detector_snr.complex_matched_filter_snr)
-                    optimal_snrs[ifo.name].append(
-                        per_detector_snr.optimal_snr_squared.real ** 0.5)
-
-            for ifo in likelihood.interferometers:
-                sample['{}_matched_filter_snr'.format(ifo.name)] =\
-                    matched_filter_snrs[ifo.name]
-                sample['{}_optimal_snr'.format(ifo.name)] =\
-                    optimal_snrs[ifo.name]
+                sample[mf_key] = matched_filter_snrs
+                sample[optimal_key] = optimal_snrs
 
     else:
         logger.debug('Not computing SNRs.')
 
 
+def _compute_snrs(args):
+    """A wrapper of computing the SNRs to enable multiprocessing"""
+    from ..core.sampler.base_sampler import _sampling_convenience_dump
+    likelihood = _sampling_convenience_dump.likelihood
+    ii, sample = args
+    sample = dict(sample).copy()
+    likelihood.parameters.update(sample)
+    signal_polarizations = likelihood.waveform_generator.frequency_domain_strain(
+        sample
+    )
+    snrs = list()
+    for ifo in likelihood.interferometers:
+        snrs.append(likelihood.calculate_snrs(signal_polarizations, ifo))
+    return snrs
+
+
 def generate_posterior_samples_from_marginalized_likelihood(
-        samples, likelihood, npool=1):
+        samples, likelihood, npool=1, block=10, use_cache=True):
     """
     Reconstruct the distance posterior from a run which used a likelihood which
     explicitly marginalised over time/distance/phase.
@@ -1195,16 +1476,19 @@ def generate_posterior_samples_from_marginalized_likelihood(
         Likelihood used during sampling.
     npool: int, (default=1)
         If given, perform generation (where possible) using a multiprocessing pool
+    block: int, (default=10)
+        Size of the blocks to use in multiprocessing
+    use_cache: bool, (default=True)
+        If true, cache the generation so that reconstuction can begin from the
+        cache on restart.
 
     Returns
     =======
     sample: DataFrame
         Returns the posterior with new samples.
     """
-    if not any([likelihood.phase_marginalization,
-                likelihood.distance_marginalization,
-                likelihood.time_marginalization,
-                likelihood.calibration_marginalization]):
+    marginalized_parameters = getattr(likelihood, "_marginalized_parameters", list())
+    if len(marginalized_parameters) == 0:
         return samples
 
     # pass through a dictionary
@@ -1216,23 +1500,91 @@ def generate_posterior_samples_from_marginalized_likelihood(
 
     logger.info('Reconstructing marginalised parameters.')
 
-    fill_args = [(ii, row, likelihood) for ii, row in samples.iterrows()]
+    try:
+        cache_filename = f"{likelihood.outdir}/.{likelihood.label}_generate_posterior_cache.pickle"
+    except AttributeError:
+        logger.warning("Likelihood has no outdir and label attribute: caching disabled")
+        use_cache = False
+
+    if use_cache and os.path.exists(cache_filename) and not command_line_args.clean:
+        try:
+            with open(cache_filename, "rb") as f:
+                cached_samples_dict = pickle.load(f)
+        except EOFError:
+            logger.warning("Cache file is empty")
+            cached_samples_dict = None
+
+        # Check the samples are identical between the cache and current
+        if (cached_samples_dict is not None) and (cached_samples_dict["_samples"].equals(samples)):
+            # Calculate reconstruction percentage and print a log message
+            nsamples_converted = np.sum(
+                [len(val) for key, val in cached_samples_dict.items() if key != "_samples"]
+            )
+            perc = 100 * nsamples_converted / len(cached_samples_dict["_samples"])
+            logger.info(f'Using cached reconstruction with {perc:0.1f}% converted.')
+        else:
+            logger.info("Cached samples dict out of date, ignoring")
+            cached_samples_dict = dict(_samples=samples)
+
+    else:
+        # Initialize cache dict
+        cached_samples_dict = dict()
+
+        # Store samples to convert for checking
+        cached_samples_dict["_samples"] = samples
+
+    # Set up the multiprocessing
     if npool > 1:
-        pool = multiprocessing.Pool(processes=npool)
+        from ..core.sampler.base_sampler import _initialize_global_variables
+        pool = multiprocessing.Pool(
+            processes=npool,
+            initializer=_initialize_global_variables,
+            initargs=(likelihood, None, None, False),
+        )
         logger.info(
             "Using a pool with size {} for nsamples={}"
             .format(npool, len(samples))
         )
-        new_samples = np.array(pool.map(fill_sample, tqdm(fill_args, file=sys.stdout)))
-        pool.close()
     else:
-        new_samples = np.array([fill_sample(xx) for xx in tqdm(fill_args, file=sys.stdout)])
+        from ..core.sampler.base_sampler import _sampling_convenience_dump
+        _sampling_convenience_dump.likelihood = likelihood
+        pool = None
+
+    fill_args = [(ii, row) for ii, row in samples.iterrows()]
+    ii = 0
+    pbar = tqdm(total=len(samples), file=sys.stdout)
+    while ii < len(samples):
+        if ii in cached_samples_dict:
+            ii += block
+            pbar.update(block)
+            continue
+
+        if pool is not None:
+            subset_samples = pool.map(fill_sample, fill_args[ii: ii + block])
+        else:
+            subset_samples = [list(fill_sample(xx)) for xx in fill_args[ii: ii + block]]
+
+        cached_samples_dict[ii] = subset_samples
+
+        if use_cache:
+            with open(cache_filename, "wb") as f:
+                pickle.dump(cached_samples_dict, f)
+
+        ii += block
+        pbar.update(len(subset_samples))
+    pbar.close()
+
+    if pool is not None:
+        pool.close()
+        pool.join()
+
+    new_samples = np.concatenate(
+        [np.array(val) for key, val in cached_samples_dict.items() if key != "_samples"]
+    )
+
+    for ii, key in enumerate(marginalized_parameters):
+        samples[key] = new_samples[:, ii]
 
-    samples['geocent_time'] = new_samples[:, 0]
-    samples['luminosity_distance'] = new_samples[:, 1]
-    samples['phase'] = new_samples[:, 2]
-    if likelihood.calibration_marginalization:
-        samples['recalib_index'] = new_samples[:, 3]
     return samples
 
 
@@ -1257,14 +1609,11 @@ def generate_sky_frame_parameters(samples, likelihood):
 
 
 def fill_sample(args):
-    ii, sample, likelihood = args
+    from ..core.sampler.base_sampler import _sampling_convenience_dump
+    likelihood = _sampling_convenience_dump.likelihood
+    ii, sample = args
+    marginalized_parameters = getattr(likelihood, "_marginalized_parameters", list())
     sample = dict(sample).copy()
     likelihood.parameters.update(dict(sample).copy())
     new_sample = likelihood.generate_posterior_sample_from_marginalized_likelihood()
-
-    if not likelihood.calibration_marginalization:
-        return new_sample["geocent_time"], new_sample["luminosity_distance"],\
-            new_sample["phase"]
-    else:
-        return new_sample["geocent_time"], new_sample["luminosity_distance"],\
-            new_sample["phase"], new_sample['recalib_index']
+    return tuple((new_sample[key] for key in marginalized_parameters))
