@@ -1,3 +1,7 @@
+"""
+Wrappers to :code:`Astropy` functionality for specifying the cosmology to use.
+"""
+
 DEFAULT_COSMOLOGY = None
 COSMOLOGY = [None, str(None)]
 
@@ -11,16 +15,6 @@ def _set_default_cosmology():
 
 
 def get_cosmology(cosmology=None):
-    from astropy import cosmology as cosmo
-    _set_default_cosmology()
-    if cosmology is None:
-        cosmology = DEFAULT_COSMOLOGY
-    elif isinstance(cosmology, str):
-        cosmology = cosmo.__dict__[cosmology]
-    return cosmology
-
-
-def set_cosmology(cosmology=None):
     """
     Get an instance of a astropy.cosmology.FLRW subclass.
 
@@ -34,8 +28,6 @@ def set_cosmology(cosmology=None):
             None - Use DEFAULT_COSMOLOGY
             Instance of astropy.cosmology.FLRW subclass
             String with name of known Astropy cosmology, e.g., "Planck13"
-            Dictionary with arguments required to instantiate the cosmology
-            class.
 
     Returns
     =======
@@ -44,12 +36,36 @@ def set_cosmology(cosmology=None):
     """
     from astropy import cosmology as cosmo
     _set_default_cosmology()
+    if cosmology is None:
+        cosmology = DEFAULT_COSMOLOGY
+    elif isinstance(cosmology, str):
+        cosmology = getattr(cosmo, cosmology)
+    return cosmology
+
+
+def set_cosmology(cosmology=None):
+    """
+    Set an instance of a astropy.cosmology.FLRW subclass as the default
+    cosmology.
+
+    Parameters
+    ==========
+    cosmology: astropy.cosmology.FLRW, str, dict
+        Description of cosmology, one of:
+            None - Use DEFAULT_COSMOLOGY
+            Instance of astropy.cosmology.FLRW subclass
+            String with name of known Astropy cosmology, e.g., "Planck13"
+            Dictionary with arguments required to instantiate the cosmology
+            class.
+    """
+    from astropy import cosmology as cosmo
+    _set_default_cosmology()
     if cosmology is None:
         cosmology = DEFAULT_COSMOLOGY
     elif isinstance(cosmology, cosmo.FLRW):
         cosmology = cosmology
     elif isinstance(cosmology, str):
-        cosmology = cosmo.__dict__[cosmology]
+        cosmology = getattr(cosmo, cosmology)
     elif isinstance(cosmology, dict):
         if 'Ode0' in cosmology.keys():
             if 'w0' in cosmology.keys():
@@ -63,3 +79,15 @@ def set_cosmology(cosmology=None):
         COSMOLOGY[1] = cosmology.name
     else:
         COSMOLOGY[1] = repr(cosmology)
+
+
+def z_at_value(func, fval, **kwargs):
+    """
+    Wrapped version of :code:`astropy.cosmology.z_at_value` to return float
+    rather than an :code:`astropy Quantity` as returned for :code:`astropy>=5`.
+
+    See https://docs.astropy.org/en/stable/api/astropy.cosmology.z_at_value.html#astropy.cosmology.z_at_value
+    for detailed documentation.
+    """
+    from astropy.cosmology import z_at_value
+    return z_at_value(func=func, fval=fval, **kwargs).value
