@@ -17,7 +17,7 @@ conversion_dict = {'pressure': {'cgs': C_SI ** 4. / G_SI * 10., 'si': C_SI ** 4.
                    'pseudo_enthalpy': {'dimensionless': 1.},
                    'mass': {'g': C_SI ** 2. / G_SI * 1000, 'kg': C_SI ** 2. / G_SI, 'geom': 1.,
                             'm_sol': C_SI ** 2. / G_SI / MSUN_SI},
-                   'radius': {'cm': 100., 'mass': 1., 'km': .001},
+                   'radius': {'cm': 100., 'm': 1., 'km': .001},
                    'tidal_deformability': {'geom': 1.}}
 
 
@@ -668,7 +668,7 @@ class EOSFamily(object):
         # If we're not at the end of the array, determine actual maximum mass. Else, assume
         # last point is the maximum mass and proceed.
         if i < (npts - 1):
-            # Now replace ith point with interpolated maximum mass
+            # Now replace with point with interpolated maximum mass
             # This is done by interpolating the last three points and then
             # minimizing the negative of the interpolated function
             x = [energy_density_grid[i - 2], energy_density_grid[i - 1], energy_density_grid[i]]
