@@ -11,11 +11,11 @@ class Likelihood(bilby.Likelihood):
         super().__init__(parameters=dict(x=None, y=None))
 
     def log_likelihood(self):
-        return -1 / (self.interp(self.parameters['x'], self.parameters['y'])[0])
+        return -1 / (self.interp(self.parameters["x"], self.parameters["y"])[0])
 
 
-for letter in ['B', 'I', 'L', 'Y']:
-    img = 1 - io.imread('{}.png'.format(letter), as_gray=True)[::-1, :]
+for letter in ["B", "I", "L", "Y"]:
+    img = 1 - io.imread("{}.png".format(letter), as_gray=True)[::-1, :]
     x = np.arange(img.shape[0])
     y = np.arange(img.shape[1])
     interp = si.interpolate.interp2d(x, y, img.T)
@@ -23,10 +23,14 @@ for letter in ['B', 'I', 'L', 'Y']:
     likelihood = Likelihood(interp)
 
     priors = {}
-    priors['x'] = bilby.prior.Uniform(0, max(x), 'x')
-    priors['y'] = bilby.prior.Uniform(0, max(y), 'y')
+    priors["x"] = bilby.prior.Uniform(0, max(x), "x")
+    priors["y"] = bilby.prior.Uniform(0, max(y), "y")
 
     result = bilby.run_sampler(
-        likelihood=likelihood, priors=priors, sampler='nestle', npoints=5000,
-        label=letter)
+        likelihood=likelihood,
+        priors=priors,
+        sampler="nestle",
+        npoints=5000,
+        label=letter,
+    )
     fig = result.plot_corner(quantiles=None)
