@@ -1,7 +1,10 @@
+import os
+import shutil
 import unittest
+from unittest import mock
 
-import mock
 import numpy as np
+import pandas as pd
 
 import bilby
 
@@ -320,6 +323,43 @@ class TestConditionalPriorDict(unittest.TestCase):
             expected.append(expected[-1] * self.test_sample[f"var_{ii}"])
         self.assertListEqual(expected, res)
 
+    def test_rescale_with_joint_prior(self):
+        """
+        Add a joint prior into the conditional prior dictionary and check that
+        the returned list is flat.
+        """
+
+        # set multivariate Gaussian distribution
+        names = ["mvgvar_0", "mvgvar_1"]
+        mu = [[0.79, -0.83]]
+        cov = [[[0.03, 0.], [0., 0.04]]]
+        mvg = bilby.core.prior.MultivariateGaussianDist(names, mus=mu, covs=cov)
+
+        priordict = bilby.core.prior.ConditionalPriorDict(
+            dict(
+                var_3=self.prior_3,
+                var_2=self.prior_2,
+                var_0=self.prior_0,
+                var_1=self.prior_1,
+                mvgvar_0=bilby.core.prior.MultivariateGaussian(mvg, "mvgvar_0"),
+                mvgvar_1=bilby.core.prior.MultivariateGaussian(mvg, "mvgvar_1"),
+            )
+        )
+
+        ref_variables = list(self.test_sample.values()) + [0.4, 0.1]
+        keys = list(self.test_sample.keys()) + names
+        res = priordict.rescale(keys=keys, theta=ref_variables)
+
+        self.assertIsInstance(res, list)
+        self.assertEqual(np.shape(res), (6,))
+        self.assertListEqual([isinstance(r, float) for r in res], 6 * [True])
+
+        # check conditional values are still as expected
+        expected = [self.test_sample["var_0"]]
+        for ii in range(1, 4):
+            expected.append(expected[-1] * self.test_sample[f"var_{ii}"])
+        self.assertListEqual(expected, res[0:4])
+
     def test_cdf(self):
         """
         Test that the CDF method is the inverse of the rescale method.
@@ -371,5 +411,33 @@ class TestConditionalPriorDict(unittest.TestCase):
         print(res)
 
 
+class TestDirichletPrior(unittest.TestCase):
+
+    def setUp(self):
+        self.priors = bilby.core.prior.DirichletPriorDict(5)
+
+    def tearDown(self):
+        if os.path.isdir("priors"):
+            shutil.rmtree("priors")
+
+    def test_samples_sum_to_less_than_one(self):
+        """
+        Test that the samples sum to less than one as required for the
+        Dirichlet distribution.
+        """
+        samples = pd.DataFrame(self.priors.sample(10000)).values
+        self.assertLess(max(np.sum(samples, axis=1)), 1)
+
+    def test_read_write_file(self):
+        self.priors.to_file(outdir="priors", label="test")
+        test = bilby.core.prior.PriorDict(filename="priors/test.prior")
+        self.assertEqual(self.priors, test)
+
+    def test_read_write_json(self):
+        self.priors.to_json(outdir="priors", label="test")
+        test = bilby.core.prior.PriorDict.from_json(filename="priors/test_prior.json")
+        self.assertEqual(self.priors, test)
+
+
 if __name__ == "__main__":
     unittest.main()
