@@ -69,6 +69,7 @@ class TestResult(unittest.TestCase):
             sampler_kwargs=dict(test="test", func=lambda x: x),
             injection_parameters=dict(x=0.5, y=0.5),
             meta_data=dict(test="test"),
+            sampling_time=100.0,
         )
 
         n = 100
@@ -134,6 +135,25 @@ class TestResult(unittest.TestCase):
         with self.assertRaises(IOError):
             bilby.core.result.read_in_result(filename="not/a/file.json")
 
+        with self.assertRaises(IOError):
+            incomplete_json = """
+{
+  "label": "label",
+  "outdir": "outdir",
+  "sampler": "dynesty",
+  "log_evidence": 0,
+  "log_evidence_err": 0,
+  "log_noise_evidence": 0,
+  "log_bayes_factor": 0,
+  "priors": {
+    "chirp_mass": {
+"""
+            with open("{}/incomplete.json".format(self.result.outdir), "wb") as ff:
+                ff.write(incomplete_json)
+            bilby.core.result.read_in_result(
+                filename="{}/incomplete.json".format(self.result.outdir)
+            )
+
     def test_unset_priors(self):
         result = bilby.core.result.Result(
             label="label",
@@ -235,6 +255,7 @@ class TestResult(unittest.TestCase):
         self.assertEqual(self.result.priors["y"], loaded_result.priors["y"])
         self.assertEqual(self.result.priors["c"], loaded_result.priors["c"])
         self.assertEqual(self.result.priors["d"], loaded_result.priors["d"])
+        self.assertEqual(self.result.sampling_time, loaded_result.sampling_time)
 
     def test_save_and_dont_overwrite_json(self):
         self._save_and_dont_overwrite_test(extension='json')
@@ -657,5 +678,38 @@ class TestMiscResults(unittest.TestCase):
         self.assertEqual(labels_checked, ["a", "$a$", "a-1", "$a_1$"])
 
 
+class TestPPPlots(unittest.TestCase):
+
+    def setUp(self):
+        priors = bilby.core.prior.PriorDict(dict(
+            a=bilby.core.prior.Uniform(0, 1, latex_label="$a$"),
+            b=bilby.core.prior.Uniform(0, 1, latex_label="$b$"),
+        ))
+        self.results = [
+            bilby.core.result.Result(
+                label=str(ii),
+                outdir='.',
+                search_parameter_keys=list(priors.keys()),
+                priors=priors,
+                injection_parameters=priors.sample(),
+                posterior=pd.DataFrame(priors.sample(500)),
+            )
+            for ii in range(10)
+        ]
+
+    def test_make_pp_plot(self):
+        _ = bilby.core.result.make_pp_plot(self.results, save=False)
+
+    def test_pp_plot_raises_error_with_wrong_number_of_lines(self):
+        with self.assertRaises(ValueError):
+            _ = bilby.core.result.make_pp_plot(self.results, save=False, lines=["-"])
+
+    def test_pp_plot_raises_error_with_wrong_number_of_confidence_intervals(self):
+        with self.assertRaises(ValueError):
+            _ = bilby.core.result.make_pp_plot(
+                self.results, save=False, confidence_interval_alpha=[0.1]
+            )
+
+
 if __name__ == "__main__":
     unittest.main()
