@@ -1,7 +1,7 @@
 import unittest
+from unittest.mock import MagicMock
 
 import numpy as np
-from mock import MagicMock
 
 import bilby
 
@@ -33,7 +33,6 @@ class TestDynesty(unittest.TestCase):
             sample="rwalk",
             periodic=None,
             reflective=None,
-            verbose=True,
             check_point_delta_t=1800,
             nlive=1000,
             first_update=None,
@@ -66,6 +65,7 @@ class TestDynesty(unittest.TestCase):
             n_effective=None,
             maxmcmc=5000,
             nact=5,
+            print_method="tqdm",
         )
         self.sampler.kwargs[
             "print_func"
@@ -89,7 +89,6 @@ class TestDynesty(unittest.TestCase):
             sample="rwalk",
             periodic=[],
             reflective=[],
-            verbose=True,
             check_point_delta_t=1800,
             nlive=1000,
             first_update=None,
@@ -122,6 +121,7 @@ class TestDynesty(unittest.TestCase):
             n_effective=None,
             maxmcmc=5000,
             nact=5,
+            print_method="tqdm",
         )
 
         for equiv in bilby.core.sampler.base_sampler.NestedSampler.npoints_equiv_kwargs:
