@@ -1,7 +1,6 @@
 import shutil
 import unittest
-
-from mock import MagicMock
+from unittest.mock import MagicMock
 
 import bilby
 
@@ -29,7 +28,7 @@ class TestUltranest(unittest.TestCase):
 
     def test_default_kwargs(self):
         expected = dict(
-            resume=True,
+            resume="overwrite",
             show_status=True,
             num_live_points=None,
             wrapped_params=None,
@@ -64,7 +63,7 @@ class TestUltranest(unittest.TestCase):
 
     def test_translate_kwargs(self):
         expected = dict(
-            resume=True,
+            resume="overwrite",
             show_status=True,
             num_live_points=123,
             wrapped_params=None,
